@@ -17,7 +17,7 @@ my $sth;#sth hub
 
 =head1 WARN
 
-Still incomplete!
+More or less complete!
 
 =head1 NAME
 
@@ -49,8 +49,6 @@ Both above options determining the module controller for web actions on tables r
 
 =item * B<trust> is a url subprefix for trust admin urls of this module. See defaults below.
 
-=item * B<schema> is a Postgresql schema name.
-
 =back
 
 =head2 Defaults
@@ -144,7 +142,7 @@ TXT
 sub sign {
   my $c = shift;
   
-  $c->authenticate($c->stash('login') || $c->param('login'), $c->stash('pass') || $c->param('pass'))
+  $c->authenticate($c->vars('login','pass'))
     and $c->redirect_to("admin home")
     #~ and $c->render(format=>'txt', text=>__PACKAGE__ . "\n\nSuccessfull signed! ".$c->dumper( $c->auth_user))
     and return;
@@ -165,7 +163,7 @@ sub signout {
 sub new_user {
   my $c = shift;
   
-  my ($login, $pass) = ($c->stash('login') || $c->param('login'), $c->stash('pass') ||  $c->param('pass'));
+  my ($login, $pass) = $c->vars('login', 'pass');
   
   my $r;
   ($r = $dbh->selectrow_hashref($sth->sth('user'), undef, (undef, $login)))
@@ -203,24 +201,21 @@ sub trust_new_user {
   $rl ||= $dbh->selectrow_hashref($sth->sth('new role'), undef, ('admin'));
   
   # REF role->user
-  my $ru = $dbh->selectrow_hashref($sth->sth('ref'), undef, ($rl->{id}, $u->{id}));
-  $ru ||= $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($rl->{id}, $u->{id}));
+  my $ru = $c->ref($rl->{id}, $u->{id});
   
   # CONTROLLER
   my $cc = $dbh->selectrow_hashref($sth->sth('controller'), undef, (($init_conf->{controller}, $init_conf->{namespace}) x 2,));
   $cc ||= $dbh->selectrow_hashref($sth->sth('new controller'), undef, ($init_conf->{controller}, 'admin actions'));
   
   #Namespace
-  my $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($init_conf->{namespace},));
+  my $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, (undef, $init_conf->{namespace},));
   $ns ||= $dbh->selectrow_hashref($sth->sth('new namespace'), undef, ($init_conf->{namespace}, 'plugin ns!'));
   
   #ref namespace -> controller
-  my $nc = $dbh->selectrow_hashref($sth->sth('ref'), undef, ($ns->{id}, $cc->{id}));
-  $nc ||= $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($ns->{id}, $cc->{id}));
+  my $nc = $c->ref($ns->{id}, $cc->{id});
   
   #REF namespace->role
-  my $cr = $dbh->selectrow_hashref($sth->sth('ref'), undef, ($ns->{id}, $rl->{id}));
-  $cr ||= $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($ns->{id}, $rl->{id}));
+  my $cr = $c->ref($ns->{id}, $rl->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -245,7 +240,7 @@ TXT
 
 sub new_role {
 	my $c = shift;
-	my $name = $c->stash('name');
+	my ($name) = $c->vars('name');
 	my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, (undef, $name));
 	$c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -274,7 +269,7 @@ TXT
 
 sub user_roles {
   my $c = shift;
-  my $user = $c->stash('user') || $c->param('user');
+  my ($user) = $c->vars('user');
   my $u =  $dbh->selectrow_hashref($sth->sth('user'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
   
   $c->render(format=>'txt', text=><<TXT)
@@ -308,7 +303,7 @@ TXT
 sub new_role_user {
   my $c = shift;
   
-  my $role = $c->stash('role') || $c->param('role');
+  my ($role) = $c->vars('role');
   # ROLE
   my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
   $c->render(format=>'txt', text=><<TXT)
@@ -347,7 +342,7 @@ TXT
     and return
     if $ref;
   
-  $ref = $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($r->{id}, $u->{id}));
+  $ref = $c->ref($r->{id}, $u->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -364,7 +359,7 @@ TXT
 sub del_role_user {# удалить связь пользователя с ролью
   my $c = shift;
   
-  my $role = $c->stash('role') || $c->param('role');
+  my ($role) = $c->vars('role');
   # ROLE
   my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
   $c->render(format=>'txt', text=><<TXT)
@@ -416,7 +411,7 @@ sub disable_role {
   my $a = shift // 1; # 0-enable 1 - disable
   my $k = {0=>'enable', 1=>'disable',};
   
-  my $role = $c->stash('role') || $c->param('role');
+  my ($role) = $c->vars('role');
   # ROLE
   my $r = $dbh->selectrow_hashref($sth->sth('dsbl/enbl role'), undef, ($a, $role =~ /\D/ ? (undef, $role) : ($role, undef,)));
   $c->render(format=>'txt', text=><<TXT)
@@ -446,7 +441,7 @@ sub enable_role {shift->disable_role(0);}
 sub role_users {# все пользователи роли по запросу /myadmin/users/:role
   my $c = shift;
   
-  my $role = $c->stash('role') || $c->param('role');
+  my ($role) = $c->vars('role');
   # ROLE
   my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
   $c->render(format=>'txt', text=><<TXT)
@@ -473,7 +468,7 @@ TXT
 sub role_routes {# все маршруты роли по запросу /myadmin/routes/:role
   my $c = shift;
   
-   my $role = $c->stash('role') || $c->param('role');
+   my ($role) = $c->vars('role');
   # ROLE
   my $r = $dbh->selectrow_hashref($sth->sth('role'), undef, ($role =~ /\D/ ? (undef, $role) : ($role, undef,)));
   $c->render(format=>'txt', text=><<TXT)
@@ -501,7 +496,21 @@ sub controllers {
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-CONTROLLERS TABLE (@{[scalar @$list]})
+CONTROLLERS (@{[scalar @$list]})
+===
+
+@{[$c->dumper( $list)]}
+TXT
+}
+
+sub controller {# /controller/:ns/:controll
+  my $c = shift;
+  my ($ns, $controll) = $c->vars(qw(ns controll));
+  my $list = $dbh->selectall_arrayref($sth->sth('controller', where=>"where (id=? or controller=?) and (namespace_id = ? or namespace = ? or (?::varchar is null and namespace is null))"), { Slice => {} }, ($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns));
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+CONTROLLER (@{[scalar @$list]})
 ===
 
 @{[$c->dumper( $list)]}
@@ -510,9 +519,9 @@ TXT
 
 sub new_controller {
   my $c = shift;
-  my $ns = $c->stash('ns') || $c->param('ns') ||  $c->stash('namespace') || $c->param('namespace');
-  $ns = undef if $ns eq 'undef';
-  my $mod = $c->stash('module') || $c->param('module');
+  #~ my $ns = $c->stash('ns') || $c->param('ns') ||  $c->stash('namespace') || $c->param('namespace');
+  my ($ns) = $c->vars('ns') || $c->vars('namespace');
+  my ($mod) = $c->vars('module');
   my $cn = $dbh->selectrow_hashref($sth->sth('controller'), undef, ($mod, ($ns) x 2,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -526,7 +535,7 @@ TXT
   if $cn;
   my $n = $c->new_namespace($ns) if $ns;
   $cn = $dbh->selectrow_hashref($sth->sth('new controller'), undef, ($mod, undef));
-  $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($n->{id}, $cn->{id}))
+  $c->ref($n->{id}, $cn->{id})
     if $n;
   
   $cn = $dbh->selectrow_hashref($sth->sth('controller'), undef, ($mod, ($ns) x 2,));
@@ -544,8 +553,8 @@ TXT
 
 sub new_namespace {
   my $c = shift;
-  my $ns = shift ||  $c->stash('ns') || $c->param('ns') ||  $c->stash('namespace') || $c->param('namespace');
-  my $n = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns));
+  my ($ns) = shift ||  $c->vars('ns') || $c->vars('namespace');
+  my $n = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -572,6 +581,9 @@ TXT
 sub actions {
   my $c = shift;
   my $list = $dbh->selectall_arrayref($sth->sth('actions'), { Slice => {} }, );
+  map {
+    $_->{routes} = $dbh->selectall_arrayref($sth->sth('action routes', where=>"where action_id=?"), { Slice => {} }, ($_->{id}));
+  } @$list;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -585,6 +597,7 @@ TXT
 sub routes {
   my $c = shift;
   my $list = $dbh->selectall_arrayref($sth->sth('apply routes'), { Slice => {} }, );
+  
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -595,6 +608,203 @@ ROUTES list (@{[scalar @$list]})
 TXT
 }
 
+sub new_route_ns {# показать список мест-имен
+  my $c = shift;
+  my $list = $dbh->selectall_arrayref($sth->sth('namespaces'), { Slice => {} }, );
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+1. Для нового маршрута укажите имя namespace или его ID или undef.
+Новый можно ввести.
+
+Namespaces (@{[scalar @$list]})
+===
+
+@{[$c->dumper( $list )]}
+TXT
+}
+
+sub new_route_c {# показать список контроллеров
+  my $c = shift;
+  my ($ns) = $c->vars('ns');
+  $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
+    || {namespace => $ns};
+  my $list = $dbh->selectall_arrayref($sth->sth('controllers', where=>"where n.id=? or (?::int is null and n.id is null)"), { Slice => {} }, ($ns->{id}, $ns->{id}));
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+1. namespace = [$ns->{id}:$ns->{namespace}]
+
+Указать имя или ID контроллера или ввести новое имя
+
+Controllers (@{[scalar @$list]})
+===
+
+@{[$c->dumper( $list )]}
+TXT
+}
+
+sub new_route_a {# показать список действий
+  my $c = shift;
+  my ($ns, $controll) = $c->vars('ns', 'controll');
+  
+  $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
+    || {namespace => $ns};
+  
+  $controll = $dbh->selectrow_hashref($sth->sth('controllers', where=>"where (n.id=? or (?::varchar is null and n.id is null)) and (c.id=? or c.controller=?)"), undef, ($ns->{id}, $ns->{id}, $controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), ))
+    || {controller=>$controll};
+  
+  my $list = $dbh->selectall_arrayref($sth->sth('actions', where=>"where controller_id=?"), { Slice => {} }, ($controll->{id}));
+  my $list2 = $dbh->selectall_arrayref($sth->sth('actions', where=>"where controller_id is null"), { Slice => {} }, ());
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+1. namespace = [$ns->{id}:$ns->{namespace}]
+2. controller = [$controll->{id}:$controll->{controller}]
+
+Указать имя или ID действия из списка или ввести новое имя действия
+
+Actions for selected controller (@{[scalar @$list]})
+===
+@{[$c->dumper( $list )]}
+
+Actions without controller (@{[scalar @$list2]}):
+===
+@{[$c->dumper( $list2 )]}
+
+TXT
+}
+
+my @route_cols = qw(request name descr auth disable order_by);
+sub new_route {# показать маршруты к действию
+  my $c = shift;
+  my ($ns, $controll, $act) = $c->vars('ns', 'controll', 'act');
+  
+  $ns = $dbh->selectrow_hashref($sth->sth('namespace'), undef, ($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
+    || {namespace => $ns};
+  
+  $controll = $dbh->selectrow_hashref($sth->sth('controllers', where=>"where (n.id=? or (?::varchar is null and n.id is null)) and (c.id=? or c.controller=?)"), undef, ($ns->{id}, $ns->{id}, $controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), ))
+    || {controller=>$controll};
+  
+  $act = $dbh->selectrow_hashref($sth->sth('actions', where=>"where controller_id=? and (a.id = ? or a.action = ? )"), undef, ($controll->{id}, $act =~ /\D/ ? (undef, $act) : ($act, undef,),))
+    || $dbh->selectrow_hashref($sth->sth('actions', where=>"where controller_id is null and (a.id = ? or a.action = ? )"), undef, ($act =~ /\D/ ? (undef, $act) : ($act, undef,),))
+    || {action => $act};
+  
+  # Проверка на похожий $request ?? TODO
+  my $route = {};
+  @$route{@route_cols, 'id'} = $c->vars(@route_cols, 'id',);
+
+  my @save = ();
+  ($route->{id} || ($route->{request} && $route->{name}))
+    && (@save = $c->route_save($ns, $controll, $act, $route))
+    && $c->render(format=>'txt', text=><<TXT)
+$pkg
+
+Success done save!
+
+Namespace:
+===
+@{[$c->dumper( $save[0] )]}
+
+Controller:
+===
+@{[$c->dumper( $save[1] )]}
+
+Action:
+===
+@{[$c->dumper( $save[2] )]}
+
+Route:
+===
+@{[$c->dumper( $save[3] )]}
+
+Refs:
+===
+@{[$c->dumper( $save[4] )]}
+
+TXT
+    && return $c
+  ;
+  
+  
+  # маршруты действия
+  my $list = $act->{id} ? $dbh->selectall_arrayref($sth->sth('action routes', where=>'where action_id=?'), { Slice => {} }, ($act->{id}))
+    : [];
+  # свободные маршруты
+  my $list2 = $act->{id} ? $dbh->selectall_arrayref($sth->sth('action routes', where=>'where action_id is null'), { Slice => {} }, ())
+    : [];
+  
+  $c->render(format=>'txt', text=><<TXT);
+$pkg
+
+1. namespace = [$ns->{id}:$ns->{namespace}]
+2. controller = [$controll->{id}:$controll->{controller}]
+3. action = [$act->{id}:$act->{action}]
+
+Указано: 
+@{[map ("$_=$route->{$_}\n", @route_cols)]}
+
+Указать параметры маршрута (?request=/x/y/:z&name=xyz&descr=...):
+
+* request (request=GET POST /foo/:bar)
+* name (name=foo_bar)
+- descr (descr=пояснение такое)
+- auth (auth=1) (auth='only')
+- disable (disable=1)
+- order_by (order_by=123)
+
+Exists routes for selected action (@{[$list ? scalar @$list : 0]})
+===
+@{[$c->dumper( $list )]}
+
+Free routes (@{[$list2 ? scalar @$list2 : 0]})
+===
+@{[$c->dumper( $list2 )]}
+
+TXT
+  
+}
+
+sub route_save {
+  my $c = shift;
+  my ($ns, $controll, $act, $route) = @_;
+  local $dbh->{AutoCommit} = 0;
+  $ns = $dbh->selectrow_hashref($sth->sth('new namespace'), undef, (@$ns{qw(namespace descr)}))
+    if $ns->{namespace} && ! $ns->{id};
+  $controll = $dbh->selectrow_hashref($sth->sth('new controller'), undef, (@$controll{qw(controller descr)}))
+    unless $controll->{id};
+  $act = $dbh->selectrow_hashref($sth->sth('new action'), undef, (@$act{qw(action callback descr)}))
+    unless $act->{id};
+    
+  $route = $dbh->selectrow_hashref($sth->sth('new route'), undef, (@$route{@route_cols}))
+    unless $route->{id};
+  my $ref = [map {
+    $c->ref($$_[0]{id}, $$_[1]{id},) if $$_[0]{id} && $$_[1]{id};
+  } ([$ns, $controll], [$controll, $act], [$route, $act],)];
+  $dbh->commit;
+  return ($ns, $controll, $act, $route, $ref);
+
+}
+
+sub vars {# получить из stash || param
+  my $c = shift;
+
+  return map {
+    my $var = $c->stash($_) || $c->param($_);
+    $var = undef if defined($var) && $var eq 'undef';
+    $var;
+  } @_;
+}
+
+sub ref {# get or save
+  my $c = shift;
+  my ($id1, $id2) = @_;
+    #~ $c->app->log->debug($c->dumper(\@_));
+  $dbh->selectrow_hashref($sth->sth('ref'), undef, ($id1, $id2,))
+    || $dbh->selectrow_hashref($sth->sth('new ref'), undef, ($id1, $id2,));
+}
+
+
 
 my @self_routes_cols = qw(request action name auth descr);
 sub self_routes {# from plugin!
@@ -604,11 +814,16 @@ sub self_routes {# from plugin!
 
   my $t = <<TABLE;
 /$prefix	index	admin home	1	View main page
-
+#
+# Namespaces, controllers, actions
+#
 /$prefix/controllers	controllers	$prefix controllers	1	Controllers list
 /$prefix/controller/new/:ns/:module	new_controller	$prefix new_controller	1	Add new controller by :ns and :module
+/$prefix/controller/:ns/:controll	controller	$prefix controller	1	View a controller (ID and name for NS and controller)
 /$prefix/actions	actions	$prefix actions	1	Actions list
-
+#
+# Роли и доступ
+#
 /$prefix/role/new/:name	new_role	$prefix create role	1	Add new role by :name
 /$prefix/role/del/:role/:user	del_role_user	$prefix del ref role->user	1	Delete ref :user -> :role by user.id|user.login and role.id|role.name.
 /$prefix/role/dsbl/:role	disable_role	$prefix disable role->user	1	Disable :role by role.id|role.name.
@@ -616,22 +831,33 @@ sub self_routes {# from plugin!
 /$prefix/roles	roles	$prefix view roles	1	View roles table
 /$prefix/roles/:user	user_roles	$prefix roles of user	1	View roles of :user by id|login
 /$prefix/role/:role/:user	new_role_user	$prefix create ref role->user	1	Assign :user to :role by user.id|user.login and role.id|role.name.
-
-/$prefix/route/new	new_route	$prefix create route	1	Add new route by params: request,namespace, controller,....
+#
+# Последовательный ввод нового маршрута
+#
+/$prefix/route/new	new_route_ns	$prefix create route step ns	1	Step namespace
+/$prefix/route/new/:ns	new_route_c	$prefix create route step controll	1	Step controller
+/$prefix/route/new/:ns/:controll	new_route_a	$prefix create route step action	1	Step action
+/$prefix/route/new/:ns/:controll/:act	new_route	$prefix create route step request	1	Step request. Params: request, name, auth, descr, ....
+/$prefix/route/new/:ns/:controll/:act/:id	new_route	$prefix create route step exist route	1	Step by route id to assign to ns-controller-action
+#
+# Маршруты и доступ
+#
 /$prefix/routes	routes	$prefix view routes	1	View routes list
 /$prefix/routes/:role	role_routes	$prefix routes of role	1	All routes of :role by id|name
 /$prefix/route/:route/:role	ref	$prefix create ref route->role	1	Assign :route with :role by route.id and role.id|role.name
-
+#
+# Пользователи
+#
 /$prefix/user/new	new_user	$prefix create user	1	Add new user by params: login,pass,...
 /$prefix/user/new/:login/:pass	new_user	$prefix create user st	1	Add new user by :login & :pass
 /$prefix/users	users	$prefix view users	1	View users table
 /$prefix/users/:role	role_users	$prefix users of role	1	View users of :role by id|name
-
+#
 get foo /sign/in	sign	signin form	0	Login&pass form
 post /sign/in	sign	signin params	0	Auth by params
 /sign/in/:login/:pass	sign	signin stash	0	Auth by stash
 /sign/out	signout	go away	1	Exit
-
+#
 /$prefix/$trust/admin/new/:login/:pass	trust_new_user	$prefix/$trust !trust create user!	0	Add new user by :login & :pass and auto assign to role 'Admin' and assign to access this controller!
 
 TABLE
