@@ -63,12 +63,17 @@ L<DBIx::POS::Template>
   from {% $schema %}routes r
     join {% $schema %}refs rf on r.id=rf.id1
     join 
-    (select a.*, c.controller, c.id as controller_id, n.namespace, n.id as namespace_id
+    (
+      select a.*, c.*
       from {% $schema %}actions a 
-      left join {% $schema %}refs r on a.id=r.id2
-      left join {% $schema %}controllers c on c.id=r.id1
-      left join {% $schema %}refs r2 on c.id=r2.id2
-      left join {% $schema %}namespaces n on n.id=r2.id1
+      left join (
+        select r.id2 as _id, c.controller, c.id as controller_id, n.namespace, n.id as namespace_id
+        from 
+          {% $schema %}refs r
+          join {% $schema %}controllers c on r.id1=c.id
+          left join {% $schema %}refs r2 on c.id=r2.id2
+          left join {% $schema %}namespaces n on n.id=r2.id1
+      ) c on a.id=c._id
     ) ac on rf.id2=ac.id
   order by r.order_by, r.ts;
 
@@ -103,7 +108,7 @@ L<DBIx::POS::Template>
 
 =name access action
 
-=desc доступ к действию
+=desc доступ к действию в контроллере (действие-каллбак - доступ проверяется по его ID)
 
 =sql
 
@@ -250,16 +255,14 @@ L<DBIx::POS::Template>
 
 =sql
 
+  select * from (
   select c.*, n.namespace, n.id as namespace_id, n.descr as namespace_descr
   from
     {% $schema %}controllers c
     left join {% $schema %}refs r on c.id=r.id2
     left join {% $schema %}namespaces n on n.id=r.id1
-  
-  where
-    c.controller=?
-    and (n.namespace=? or (?::varchar is null and n.id is null))
-    
+  ) s
+  {% $where %}
 
 =item * B<new controller> 
 
@@ -273,6 +276,27 @@ L<DBIx::POS::Template>
   values (?,?)
   returning *;
 
+=item * B<action routes> 
+
+=name action routes
+
+=desc маршрут может не привязан к действию
+
+=sql
+
+  select * from (
+  select r.*, s.action_id
+  from {% $schema %}routes r
+    left join (
+     select s.id1, a.id as action_id
+     from {% $schema %}refs s
+      join {% $schema %}actions a on a.id=s.id2
+    ) s on r.id=s.id1
+  ) s
+  {% $where %}; -- action_id is null - free routes; or action(id) routes
+  ;
+
+
 =item * B<new route> 
 
 =name new route
@@ -281,8 +305,8 @@ L<DBIx::POS::Template>
 
 =sql
 
-  insert into {% $schema %}routes (request, name, auth, descr, disable, order_by)
-  values (?,?,?,?,?,?,?,?,?)
+  insert into {% $schema %}routes (request, name, descr, auth, disable, order_by)
+  values (?,?,?,?,?,?)
   returning *;
 
 =item * B<role users> 
@@ -303,7 +327,7 @@ L<DBIx::POS::Template>
 
 =name role routes
 
-=desc Маршруты роли
+=desc Маршруты роли/действия
 
 =sql
 
@@ -326,7 +350,7 @@ L<DBIx::POS::Template>
     from {% $schema %}controllers c
     left join {% $schema %}refs r on c.id=r.id2
     left join {% $schema %}namespaces n on n.id=r.id1
-    ;
+    {% $where %};
 
 =item * B<namespaces>
 
@@ -349,7 +373,7 @@ L<DBIx::POS::Template>
 
   select *
   from {% $schema %}namespaces
-  where namespace = ?;
+  where id=? or namespace = ?;
 
 =item * B<new namespace>
 
@@ -367,14 +391,34 @@ L<DBIx::POS::Template>
 
 =name actions
 
-=desc Список
+=desc Список действий
 
 =sql
 
-  select a.*, c.namespace, c.controller
+  select * from (
+  select a.*, ac.controller_id, ac.controller
   from {% $schema %}actions a
-    left join {% $schema %}refs r on a.id=r.id2
-    left join {% $schema %}controllers c on c.id=r.id1
+    left join (
+      select a.id, c.id as controller_id, c.controller
+      from {% $schema %}actions a
+        join {% $schema %}refs r on a.id=r.id2
+        join {% $schema %}controllers c on c.id=r.id1
+      ) ac on a.id=ac.id-- действия с контроллером
+  ) as a
+  {% $where %}
+
+=item * B<new action>
+
+=name new action
+
+=desc 
+
+=sql
+
+  insert into {% $schema %}actions (action, callback, descr)
+  values (?,?,?)
+  returning *;
+
 
 =item * B<тест> 
 
